!Module of utility functions
!****************************
!Marina Pavan
!May 16, 2005
!****************************


MODULE mod_functions

USE mod_types
USE mod_computation
USE mod_param

implicit none



contains



!UTILITY FUNCTION FOR WORKER HOMEOWNERS
!***************************************

FUNCTION utility(tip,c,ho,inc,age)

use mod_types
use mod_computation
use mod_param

implicit none

!variables:
!**********

!INPUT:
!******
real(dp),intent(in)::c,ho,inc	!consumption,house,income
integer,intent(in)::age,tip

!OUTPUT:
!*******
real(dp)::utility		!utility			  


if (sigma(tip)==1.0_dp) then
    utility=itau(age)*dlog(dmax1(c,cmin))+whouse(age)*dlog(dmax1(ho,cmin))+tau(age)*dlog(tau(age)/inc)
else
    utility=((dmax1(c,cmin)**itau(age)*(dmax1(ho,cmin))**whouse(age)*(tau(age)/inc)**tau(age))**isigma(tip))/isigma(tip)-1.0_dp/isigma(tip)
endif
	

END FUNCTION utility


!UTILITY FUNCTION FOR WORKER TENANTS
!***************************************

FUNCTION utility_ten(tip,c,hprice,inc,age)

use mod_types
use mod_computation
use mod_param

implicit none

!variables:
!**********

!INPUT:
!******
real(dp),intent(in)::c,hprice,inc	!consumption,rental price,income
integer,intent(in)::age,tip

!OUTPUT:
!*******
real(dp)::utility_ten		!utility			  

if (sigma(tip)==1.0_dp) then
    utility_ten=(itau(age)+whouse(age))*dlog(dmax1(c,cmin))+tau(age)*dlog(tau(age)/inc)+whouse(age)*dlog(wrent(age)*whouse(age)/hprice)
else
    utility_ten=((dmax1(c,cmin)**(itau(age)+whouse(age))*(wrent(age)*whouse(age)/hprice)**whouse(age)*(tau(age)/inc)**tau(age))**isigma(tip))/isigma(tip)-1.0_dp/isigma(tip)
endif


    
END FUNCTION utility_ten


!UTILITY FUNCTION FOR RETIRED HOUSE OWNERS
!******************************************

FUNCTION utility_ret(tip,c,ho,age)

use mod_types
use mod_computation
use mod_param

implicit none

!variables:
!**********

!INPUT:
!******
real(dp),intent(in)::c,ho	!consumption,house
integer,intent(in)::age,tip

!OUTPUT:
!*******
real(dp)::utility_ret		!utility			  


if (sigma(tip)==1.0_dp) then
    utility_ret=dlog(dmax1(c,cmin))+whouse(age)*dlog(dmax1(ho,cmin))+tau(age)*dlog(lbar)
else
	utility_ret=((dmax1(c,cmin)*(dmax1(ho,cmin))**whouse(age)*lbar**tau(age))**isigma(tip))/isigma(tip)-1.0_dp/isigma(tip)
endif
	

END FUNCTION utility_ret


!UTILITY FUNCTION FOR RETIRED TENANTS
!******************************************

FUNCTION utility_ret_ten(tip,c,hprice,age)

use mod_types
use mod_computation
use mod_param

implicit none

!variables:
!**********

!INPUT:
!******
real(dp),intent(in)::c,hprice	!consumption
integer,intent(in)::age,tip

!OUTPUT:
!*******
real(dp)::utility_ret_ten		!utility			  

!weight on optimal c is (1+j)=1+whouse(age) 
!utility_ret_ten=iiwhouse(age)*dlog(dmax1(c,cmin))+whouse(age)*(dlog(wrent(age)*whouse(age))-dlog(hprice))

if (sigma(tip)==1.0_dp) then
    utility_ret_ten=iiwhouse(age)*dlog(dmax1(c,cmin))+whouse(age)*dlog(wrent(age)*whouse(age)/hprice)+tau(age)*dlog(lbar)
else
	utility_ret_ten=((dmax1(c,cmin)**iiwhouse(age)*(wrent(age)*whouse(age)/hprice)**whouse(age)*lbar**tau(age))**isigma(tip))/isigma(tip)-1.0_dp/isigma(tip)
endif


	

END FUNCTION utility_ret_ten




!****************************************************
!GIVES TOT PRODUCTION AS A FUNCTION OF CAPITAL
!AND LABOR AND AGGREGATE SHOCK
!F(AGS,K,L)=AGS*K^alfa*L^(1-alfa)
!****************************************************
!FUNCTION PRODUCTION(shock,Xi,LA)

!use mod_types
!use mod_computation


!implicit none

!real(dp), intent(in)::Xi  !CAPITAL, aggregate shock
!real(dp),intent(in)::LA  !aggregate labour
!integer,intent(in)::shock
!real(dp)::PRODUCTION  !optimal capital corresponding to X

!if (choice==1) then

!PRODUCTION=totlabor   !endowment economy

!else

!PRODUCTION=Ags(shock)*(Xi**alfa)*(LA**ialfa) !production economy
 
!endif 

!END FUNCTION PRODUCTION

!**********************************************
!GIVES CAPITAL AS A FUNCTION OF INTEREST RATE
!assuming F(K,L)=K^alfa*L^(1-alfa)
!K comes from optimal rule:
!R-1+deltaK=alfa*K^(alfa-1)*L^(1-alfa)
!**********************************************
!FUNCTION CAPITAL(Xi)

!use mod_types
!use mod_computation
!use mod_utils

!implicit none

!real(dp), intent(in)::Xi  !annual interest rate
!real(dp)::CAPITAL  !optimal capital corresponding to X

!if (choice==1) then

!CAPITAL=0.0_DP  !endowment economy

!else

!CAPITAL=totlabor*(alfa/(Xi-ideltaK))**iialfa   !production economy


!endif

!END FUNCTION CAPITAL

!**********************************************
!GIVES WAGE AS A FUNCTION OF AGGREGATE SHOCK
!and AGGREGATE CAPITAL
!assuming F(K,L)=AGS*K^alfa*L^(1-alfa)
!WAGE comes from optimal rule:
!WAGE=AGS*(1-alfa)*K^alfa*L^(-alfa)
!**********************************************
FUNCTION WAGE(shock,KA,LA)

use mod_types
use mod_computation


implicit none

real(dp), intent(in)::KA,LA !capital and labor
integer, intent(in)::shock
real(dp)::WAGE  !optimal WAGE corresponding to X and KA

!if (choice==1) then

!WAGE=1.0_dp   !endowment economy

!else

WAGE=ialfa*Ags(shock)*(KA**alfa)/(LA**alfa)  !production economy

!endif

END FUNCTION WAGE

!**********************************************
!GIVES GROSS INTEREST RATE AS A FUNCTION OF 
!AGGREGATE SHOCK
!and AGGREGATE CAPITAL
!assuming F(K,L)=AGS*K^alfa*L^(1-alfa)
!INTEREST comes from optimal rule:
!INTEREST=AGS*(alfa)*K^(alfa-1)*L^(1-alfa)
!**********************************************
FUNCTION INTEREST(shock,KA,LA)

use mod_types
use mod_computation


implicit none

real(dp), intent(in)::KA,LA !capital and labor
integer, intent(in)::shock
real(dp)::INTEREST  !optimal WAGE corresponding to X and KA

if (soe_dummy==0) then

INTEREST=ideltaK+alfa*Ags(shock)*KA**(-ialfa)*(LA**ialfa) 

else
    
INTEREST=interest_soe    

endif

END FUNCTION INTEREST

!**************************************
!FUNCTION TO COMPUTE LOGK'
!ON LOGK GIVEN coeff as coefficients
!**************************************
FUNCTION REGGAMMA(shock,kap)  

use mod_types
use mod_computation


real(dp),intent(in)::kap  !,house
integer, intent(in)::shock
real(dp),dimension(2)::REGGAMMA
integer::is

REGGAMMA(1)=coefK(shock)+coefK(ns+shock)*Kap
REGGAMMA(2)=coefL(shock)+coefL(ns+shock)*Kap

END FUNCTION REGGAMMA

END MODULE mod_functions


